"""
    Created on 2012-01-20
    @author: jldupont
"""
import logging, os, sys, json
from time import sleep

from jldaws.tools_date import iso_compare_string
from jldaws.tools_os import mkdir_p, resolve_path, atomic_write
from jldaws.tools_sys import retry
from jldaws.db.skv import SimpleKV

from pyfnc.fnc_tools import coroutine

def run(domain_name=None, 
        category_name=None,
        trigger_topic=None 
        ,just_key=False
        ,just_key_value=False
        ,disable_pass_through=False
        ,path_tmp=None
        ,path_dst=None
        ,keep_most_recent=None
        ,ext=None
        ,polling=None
        ,**_
        ):

    mode=path_dst and 'file' or 'stdin' 
    logging.info("Mode is: %s" % mode)

    if path_tmp is not None:
        code, path_tmp=resolve_path(path_tmp)
        if not code.startswith("ok"):
            raise Exception("Can't resolve path: %s" % path_tmp)
        mkdir_p(path_tmp)

    if path_dst is not None:
        code, path_dst=resolve_path(path_dst)
        if not code.startswith("ok"):
            raise Exception("Can't resolve path: %s" % path_dst)
        mkdir_p(path_dst)
    

    db=SimpleKV()
    def connect():
        db.connect()
    
    logging.info("Connecting to SDB... CTRL-C to abort")
    retry(connect)
    
    def create():
        db.set_domain(domain_name)
        
    logging.info("Creating domain: %s... CTRL-C to abort" % domain_name)
    retry(create)
    
    ###########
    flow_stdin=workflow_stdin(trigger_topic, db, category_name, just_key, just_key_value)
    flow_file=workflow_file(db, category_name, path_dst, path_tmp, keep_most_recent, ext)
    poll_count=0
    
    # MAIN LOOP
    ###########
    ppid=os.getppid()
    logging.info("Process pid: %s" % os.getpid())
    logging.info("Parent pid:  %s" % ppid)
    logging.info("Starting loop...")
    while True:
        if os.getppid()!=ppid:
            logging.warning("Parent terminated... exiting")
            break

        if mode is 'stdin':
            try:
                line=sys.stdin.readline().strip()
                if not disable_pass_through:
                    sys.stdout.write(line+"\n")            
            except:
                raise Exception("broken stdin/stdout...")

            flow_stdin.send(line)
            
        if mode is 'file':
            flow_file.send(poll_count)
            
            logging.debug("Sleeping...")
            sleep(polling)
            poll_count=poll_count+1
            
            
def get_batch(db, category_name):
    
    entries=[]
    next_token=None
    while True:
        try:
            logging.debug("Getting batch...")
            batch, next_token=db.get_by_category(category=category_name, next_token=next_token, last=False)
        except Exception, e:
            logging.warning(e)
            batch=None
            break
        
        if batch is None:
            break     
               
        entries.append(batch)
        
        if next_token is None:
            break
        
    return entries
    
@coroutine            
def workflow_stdin(trigger_topic, db, category_name, just_key, just_key_value):
    """for stdin trigger based"""

    while True:
        line=(yield)
        
        if trigger_topic is not None:
            if len(line)==0 or line=="":   ## safe than sorry
                continue
            
            try:
                jo=json.loads(line)
            except:
                logging.warning("Can't decode JSON from: %s" % line)
                continue
            
            try:
                topic=jo["topic"]
            except:
                logging.warning("No 'topic' field found: %s" % line)
                continue
            
            if topic!=trigger_topic:
                logging.debug("No specified topic: %s" % topic)
                continue
        
        entries=get_batch(db, category_name)
        
        for resultset in entries:
            for entry in resultset:
                
                if just_key:
                    try:    print entry["key"]
                    except:
                        logging.debug("Entry without a 'key' field...")
                        
                if just_key_value:
                    try:    print "%s\t%s" % (entry["key"], entry["value"])
                    except:
                        logging.debug("Entry without a 'key'/'value' field(s)...")
                 
                if not just_key and not just_key_value:       
                    print entry
        
    
@coroutine            
def workflow_file(db, category_name, path_dst, path_tmp, keep_most_recent, ext):
    """for file based workflow"""
    
    if ext is None:
        ext=category_name
        
    proc=process_entry(path_dst, path_tmp, keep_most_recent, ext)
    
    while True:
        _poll_count=(yield)
        
        entries=get_batch(db, category_name)
        logging.debug("Got batch...")
        
        proc.send(("begin", None))
        for resultset in entries:
            for entry in resultset:
                proc.send(("entry", entry))
        proc.send(("end", None))

    
@coroutine    
def process_entry(path_dst, path_tmp, keep_most_recent, ext):
    
    entries={}
    while True:
        code, maybe_entry=(yield)
        #logging.debug("Code: %s" % code)
        
        if code=="begin":
            logging.debug("Got 'begin'...")
            entries={}
        
        if code=="entry":
            logging.debug("Got 'entry'...")
            key=maybe_entry.get("key", None)
            if key is None:
                logging.warning("Entry doesn't have a 'key' field: %s" % maybe_entry)
            else:                
                liste=entries.get(key, [])
                liste.append(maybe_entry)
                entries[key]=liste
        
        if code=="end":
            logging.debug("Got 'end'...")
            entries=filter_entries(keep_most_recent, entries)
            write_entries(path_dst, path_tmp, ext, entries)

            
def filter_entries(keep_most_recent, entries):
    """
    For each key, keep only the latest
    (based on the 'creation_date' field)
    """
    logging.debug("Filtering entries...")
    
    if not keep_most_recent:
        return entries
    
    new_entries={}
    for key in entries:
        entry_to_keep=None
        
        for entry in entries[key]:
            if entry_to_keep is None:
                entry_to_keep=entry
                continue
            try:    
                code, delta=iso_compare_string(entry_to_keep['creation_date'], entry['creation_date'])
                if code is "ok":
                    if delta is "older":
                        entry_to_keep=entry
            except Exception,e:
                print e
        new_entries[key]=[entry_to_keep,]
            
    return new_entries


def write_entries(path_dst, path_tmp, ext, entries):
    
    logging.debug("Writing entries...")
    entries_written=0
    for entry_key in entries:

        filename=os.path.join(path_dst, "%s.%s" % (entry_key, ext))
        records=entries[entry_key]
        ## can't use .join because 'record' is really a boto.sdb.item.Item object
        contents=""
        for record in records:
            try:    contents+=json.dumps(record)
            except:
                logging.warning("Problem with JSON conversion of: %s" % str(record))
                continue

        code, msg=atomic_write(filename, contents, tmppath=path_tmp)
        if not code.startswith("ok"):
            logging.warning("Can't write to '%s': %s" % (filename, msg))
        else:
            entries_written=entries_written+1
        
    if entries_written>0:
        logging.info("Progress> wrote '%s' entries" % entries_written)
        #print contents





"""
    Created on 2012-01-20
    @author: jldupont
"""
import json
import boto
from boto.exception import S3ResponseError

def get_all_keys(bucket, prefix):
    try:
        result=[]
        
        for key in bucket.list(prefix=prefix):
            result.append(key.name)
        
        return ("ok", result)
    except:
        return ("error", "S3 access")
    
def keys_to_dict(keys):
    """
    boto.s3.keys to dictionary
    """
    d={}
    for key in keys:
        d[key.name]=key
    return d


def compute_changes(ckeys, nkeys):
    """
    Compute changes in keys
    """
    additions=[]
    substractions=[]
    changes=[]
    
    nset=set(nkeys.keys())
    cset=set(ckeys.keys())
    
    #print "cset: %s" % cset
    #print "nset: %s" % nset
    
    for k in nset:
        if k not in cset:
            additions.append(k)
            
    for k in cset:
        if k not in nset:
            substractions.append(k)
            
    common=cset.intersection(nset)
    for k in common:
        cetag=ckeys[k].etag
        netag=nkeys[k].etag
        
        if cetag != netag:
            changes.append(k)
            
    return (changes, additions, substractions)
            
    
def check_changes(ckeys, nkeys):
    """
    :param ckeys: current keys data
    :param keys:  keys data just retrieved
    """
    all_keys=set(nkeys.keys())
    all_keys.update(ckeys.keys())
    print all_keys
    changed=[]
    for k in all_keys:
        nkey=nkeys.get(k, None)
        ckey=ckeys.get(k, None)
        
        netag=nkey.etag if nkey is not None else None
        cetag=ckey.etag if ckey is not None else None
        
        print "etags:", cetag, netag
        
        if netag!=cetag:
            changed.append(k)
    return changed

def json_string(o):
    try:
        return json.dumps(o)
    except:
        return ""


KEY_FIELDS=["content_type", "content_encoding", "etag", "last_modified", "storage_class"]
def key_object_to_dict(key):
    """
    Expects a boto.s3.key.Key object type
    """
    d={}
    for k in KEY_FIELDS:
        d[k]=str(key.__dict__[k])
        
    return { key.name: d}
    

def bucket_status(bucket_name, prefix):
    
    try:
        conn=boto.connect_s3()
    except:
        raise Exception("Can't connect to S3")
    
    try:
        bucket=conn.get_bucket(bucket_name)
    except S3ResponseError:
        raise Exception("Can't find specified bucket '%s'" % bucket_name)

    try:
        keys=bucket.get_all_keys(prefix=prefix)
    except S3ResponseError:    
        pass
     
    return ("ok", keys)



"""
    Created on 2012-01-19
    @author: jldupont
"""
import logging, json, sys, os
from   time import sleep
import boto

from boto.sqs.jsonmessage import JSONMessage
from boto.sqs.message     import RawMessage

from tools_sys     import retry


def stdout(s):
    try:
        if not s.endswith("\n"):
            s=s+"\n"
    except:
        pass
    sys.stdout.write(s)
    sys.stdout.flush()

def run(queue_name=None,
        flush_queue=None,
        format_any=None,
        retry_always=None,
        topics=None,
        error_msg=None,
        simulate_error=None,
        **_
        ):
    
    queue_name=queue_name.strip()
    
    # SETUP PRIVATE QUEUE
    def setup_private_queue():
        conn = boto.connect_sqs()
        q=conn.create_queue(queue_name)
        if not format_any:
            q.set_message_class(JSONMessage)
        else:
            q.set_message_class(RawMessage)
        return q
        
    try:
        q=retry(setup_private_queue, always=retry_always)
    except Exception,e:
        raise Exception("Creating queue '%s': %s" % (queue_name, str(e)))

    if flush_queue:
        try:    
            q.clear()
            logging.info("queue flushed")
        except: pass

    ppid=os.getppid()
    logging.debug("Starting loop...")
    logging.info("Process pid: %s" % os.getpid())
    logging.info("Parent pid:  %s" % ppid)
    while True:
        if os.getppid()!=ppid:
            logging.warning("Parent terminated... exiting")
            break
        line=sys.stdin.readline()
        
        ## echo on stdout
        stdout(line)
        
        ## by default, allow the message to pass to the queue
        allow_msg=True
        if not format_any:
            try:
                jo=json.loads(line)
            except:
                logging.error("Can't load json object from: %s" % line)
                continue
            
            if topics is not None:
                try:
                    msg_topic=jo["topic"]
                except:
                    logging.error("Can't extract 'topic' field from %s: " % line)
                    continue
                try:
                    allow_msg=any(map(lambda s:msg_topic.startswith(s), topics))
                except Exception,e:
                    logging.error("Error processing message topic against list of topics: %s" % e)
                    continue
            
            if not allow_msg:
                continue
            
            m=JSONMessage()
            m.set_body(jo)
        else:
            m=RawMessage()
            m.set_body(line)
        
        logging.debug("Writing to SQS queue '%s': %s" % (queue_name, line))
        try:
            if simulate_error:
                raise Exception("Network error simulation")
            q.write(m)
        except:
            ### give us a chance...
            sleep(1)
            try:
                if simulate_error:
                    raise Exception("Network error simulation - 2nd attempt")                
                q.write(m)
            except Exception,e:
                if retry_always:
                    logging.error("Can't write to SQS queue - 2nd attempt in a row: %s" % e)
                else:
                    if error_msg is not None:
                        stdout(error_msg)
                    else:
                        raise Exception("Writing to SQS queue")

        



"""
    Created on 2012-01-20
    @author: jldupont
"""
import os, logging

from tools_os import resolve_path, split_path_version, file_contents
from tools_s3 import get_all_keys

import boto
from boto.s3.key import Key as S3Key

def filter_keys(root, keys):
    """
    Just keys beginning with 'root'
    """
    ff=lambda p: p.startswith(root)
    return (keys, filter(ff, keys))
        
def gen_key(prefix, name):
    if prefix is not None:
        return "%s/%s" % (prefix,name)
    return name
    

def run(enable_simulate=False, 
        bucket_name=None, bucket_prefix=None, 
        path_source=None, path_dest=None,
        delete_old=False,
        **_):
    
    code, path_source=resolve_path(path_source)
    if not code.startswith("ok"):
        logging.warning("Source file '%s' can't be accessed..." % path_source)
    
    
    try:
        conn = boto.connect_s3()
    except:
        ## not much we can do
        ## but actually no remote calls are made
        ## at this point so it should be highly improbable
        raise Exception("Can't 'connect' to S3")
    
    try:
        bucket=conn.create_bucket(bucket_name)
        logging.info("Got bucket '%s'" % bucket_name)        
    except:
        raise Exception("Can't get bucket '%s'" % bucket_name)
    

    base_name=os.path.basename(path_source)
    logging.info("Basename of file to upload: %s" % base_name)

    
    root_name,version,_ext=split_path_version(base_name)
    if version is not None and len(version)>0:
        logging.info("Basename of file: %s" % version)
        logging.info("Version of file:  %s" % version)
    else: 
        version=None
        
    if root_name is None:
        root_name=base_name

    if path_dest is None:
        logging.info("Will be using '%s' as filename in bucket" % base_name)
        path_dest=base_name

    key_names=None
    to_delete=None
    if delete_old:
        logging.info("Getting bucket keys")
        code, bkeys=get_all_keys(bucket, bucket_prefix)
        if not code.startswith("ok"):
            raise Exception("Can't get bucket keys...")
        
        logging.info("Got %s key(s) to filter for 'old' files" % len(bkeys))
        
        _key_names, to_delete=filter_keys(root_name, bkeys)
        
        logging.info("Older files found: %s" % to_delete)
            
    if enable_simulate:
        logging.info("! Begin simulation...")

    code, contents=file_contents(path_source)
    if not code.startswith("ok"):
        raise Exception("Can't read file '%s'" % path_source)
    
    logging.info("Got source file contents")
    
    try:
        upload_key=S3Key(bucket)
        upload_key_name=gen_key(bucket_prefix, path_dest)
        upload_key.key=upload_key_name
            
        logging.info("Prepared S3 key: %s" % upload_key.key)
    except Exception,e:
        raise Exception("S3 key generation: %s" % str(e))
    
    if enable_simulate:
        logging.info("! Would have uploaded file to S3")
    else:
        try:
            upload_key.set_contents_from_string(contents)
            logging.info("Success uploading!")
        except:
            raise Exception("Error uploading file to S3")
        
    if delete_old:
        if enable_simulate:
            logging.info("! Would have deleted the 'old' files: %s" % to_delete)
        else:
            logging.info("Deleting 'old' files:")
            for name in to_delete:
                try:
                    key=S3Key(bucket)
                    key_name=gen_key(bucket_prefix, name)
                    
                    if upload_key_name==key_name:
                        logging.info("Skipping deleting just uploaded file...")
                        continue
                    
                    #### DELETING
                    key.key=key_name
                    bucket.delete_key(key)
                    logging.info("Deleted '%s'" % key.key)
                    
                except Exception,e:
                    logging.warning("Can't delete '%s': %s" % (key.key, str(e)))
    
    logging.info("done")
    
     
    
    


"""
    Created on 2012-02-01
    @author: jldupont
"""
from tools_sys import coroutine

@coroutine
def leader_tracking():
    """
    Tracks 'leader' status - only returns transitions
    
    >>> lt=leader_tracking()
    >>> print lt.send(False)
    ('nop', None)
    >>> print lt.send(False)
    ('nop', None)
    >>> print lt.send(True)
    ('tr', 'up')
    >>> print lt.send(True)
    ('nop', None)
    >>> print lt.send(False)
    ('tr', 'down')
    >>> print lt.send(False)
    ('nop', None)    
    """
    last_status=False
    result=("nop", None)
    
    while True:
        status=(yield result)
        
        if status!=last_status:
            direction="up" if last_status==False else "down"
            last_status=status
            result=("tr", direction)
        else:
            result=("nop", None)


@coroutine
def protocol_processor(node_id, param_n, param_m):
    """
    >>> pp=protocol_processor(10, 2, 3)
    >>> pp.send((0, 11))
    (False, 'NL')
    >>> pp.send((0, 12))
    (False, 'NL')
    >>> pp.send((0, 13))
    (False, 'NL')
    >>> pp.send((1, 11))
    (False, 'NL')
    >>> pp.send((1, 12))
    (False, 'NL')
    >>> pp.send((1, 13))
    (False, 'NL')
    >>> pp.send((2, 11))
    (True, 'ML')
    >>> pp.send((2, 12))
    (False, 'ML')
    >>> pp.send((2, 13))
    (False, 'ML')
    >>> pp.send((3, 11))
    (True, 'L')
    >>> pp.send((3, 12))
    (False, 'L')
    >>> pp.send((4, 9))
    (True, 'NL')
    """
    sp=state_processor(param_n, param_m)
    
    tr=False
    current_state="NL"
    current_poll_count=0
    current_poll_winning=True
    
    while True:
        poll_count, advertised_nodeid=(yield (tr, current_state))
        tr=False
        
        ## latch on bad news...
        ## i.e. only 1 lost interval crashes the interval
        if current_poll_winning:
            current_poll_winning=(advertised_nodeid>=node_id)
        
        if current_poll_count!=poll_count:
            current_poll_count=poll_count
            tr, current_state=sp.send(current_poll_winning)
            current_poll_winning=True

@coroutine
def state_processor(param_n, param_m):
    
    last_state="NL"
    consecutive_winning_intervals=0
    current_state="NL"
    while True:

        tr=last_state != current_state
        last_state=current_state
            
        won_interval=(yield (tr, current_state))
        
        ## loose leadership easily :(
        if not won_interval:
            consecutive_winning_intervals=0
            current_state="NL"
            continue
        
        ### won this interval:    
        consecutive_winning_intervals=consecutive_winning_intervals+1

        if current_state=="NL":
            if consecutive_winning_intervals >= param_n:
                current_state="ML"
                continue
        
        if current_state=="ML":
            if consecutive_winning_intervals >= param_m:
                current_state="L"
                consecutive_winning_intervals=0
                continue
            


if __name__=="__main__":
    import doctest
    doctest.testmod()
    


"""
    Created on 2012-01-25
    @author: jldupont
"""

class SDB_Connect(Exception): pass

class SDB_CreateDomain(Exception): pass

class SDB_CreateItem(Exception): pass

class SDB_Access(Exception): pass



"""
    Created on 2012-01-20
    @author: jldupont
"""
import logging, os, sys, json

from jldaws.tools_sys import retry
from jldaws.db.skv import SimpleKV

import boto


def run(domains_list=False 
        ,domain_name=None
        ,domain_delete=False
        ,domain_create=False
        ,**_
        ):

    def connect():
        return boto.connect_sdb()
    
    logging.info("Connecting to SDB... CTRL-C to abort")
    db=retry(connect)
    
    if domains_list:
        logging.info("Getting all domains...")
        
        try:
            domains=db.get_all_domains()
            for domain in domains:
                print domain.name
        except Exception,e:
            logging.error("Can't retrieve all domains: %s" % e)
            return 
        
    if domain_delete or domain_create:
        if domain_name is None:
            logging.error("Domain name must be specified")
            return
        
    if domain_delete:
        try:
            logging.info("Deleting domain: %s" % domain_name)
            db.delete_domain(domain_name)
            logging.info("Progress: deleted domain: %s" % domain_name)
        except Exception,e:
            logging.error("Error deleting domain: %s" % e.error_message)
            return
        
    if domain_create:
        try:
            logging.info("Creating domain: %s" % domain_name)
            db.create_domain(domain_name)
            logging.info("Progress: created domain: %s" % domain_name)
        except Exception,e:
            logging.error("Error deleting domain: %s" % e.error_message)
            return
    
        


#!/usr/bin/env python
"""
    Jean-Lou Dupont's AWS scripts
    
    Created on 2012-01-19
    @author: jldupont
"""
__author__  ="Jean-Lou Dupont"
__version__ ="0.4.9"


from distutils.core import setup
from setuptools import find_packages

DESC="""
Overview
--------

This package contains a collection of Amazon Web Service related scripts e.g.

* jldsqsjob : wait for stdin trigger, dequeue msg(s) from SQS Q#1, send to stdout, requeue to SQS Q#2, delete msg(s) from Q#1
* jlds3upload : automated file upload to S3
* jlds3download : automated file download from S3
* jldrxsqs : receive from an SQS queue to JSON/string stdout (with optional stdin trigger)
* jldtxsqs : transmit on SQS JSON/string from stdin
* jldleader: distributed leader election protocol manager
* jlds3up  : simple file upload to s3 with functionality to delete 'old' files
* jlds3list : listing files in an S3 bucket
* jldsdb: list, create and delete operations on SDB
* jldsdbget: retrieve entries in SDB
* jldsdbinsert: insert records in SDB
* jldsdbbatchinsert: batch insert records in SDB from file

The philosophy behind these scripts is:

* simplicity  : each script only does 1 thing
* reporting   : status & error reporting through stderr and (optionally) JSON stdout
* data driven : object in from src ==> object out to dst
* data flow   : piping through stdin / stdout    

Scripts can easily be "composed" through standard Linux piping :

    jldrxsqs @config1.txt | some_other_command | jldtxsqs @config2.txt

The data format used is JSON.

Configuration
-------------

Can be performed through options on the command line or using a file (use a leading `@`).

History
-------

Changes history can be found at the project page: https://github.com/jldupont/jldaws
"""

import os
bp=os.path.dirname(os.path.abspath(__file__))
sp=os.path.join(bp, "src", "scripts")
scripts=os.listdir(sp)
scripts=map(lambda p:os.path.join("src", "scripts", p), scripts)


setup(name=         'jldaws',
      version=      __version__,
      description=  'Collection of Amazon AWS related scripts',
      author=       __author__,
      author_email= 'jl@jldupont.com',
      url=          'http://www.systemical.com/doc/opensource/jldaws',
      package_dir=  {'': "src",},
      packages=     find_packages("src"),

      scripts=      scripts,
      zip_safe=False
      ,long_description=DESC
      ,install_requires=["pyfnc >= 0.1.2"]
      )

#############################################

f=open("latest", "w")
f.write(str(__version__)+"\n")
f.close()



"""
    Created on 2012-01-26
    @author: jldupont
"""
import shutil, sys, json, logging
from time import sleep
import functools

from tools_logging import enable_duplicates_filter, setloglevel, info_dump, setup_syslog

class SignalTerminate(Exception): pass

def process_command_line(parser):
    parser.add_argument('-lc',  dest="logconfig", type=str,  help="Logging configuration file", choices=["debug", "info", "warning", "error"])
    parser.add_argument('-ll',  dest='log_level',     type=str,            help="Log Level", default="info", choices=["debug", "info", "warning", "error"])
    parser.add_argument('-kdf', dest='log_keepdup',   action="store_true", help="Keep duplicate log entries", default=False)
    parser.add_argument('-lsl', dest='log_syslog',    action="store_true", help="Enable syslog", default=False)
    
    args=dnorm(vars(parser.parse_args()))
    
    if (args["log_syslog"]):
        setup_syslog()
        
    if (not args["log_keepdup"]):
        enable_duplicates_filter()
        
    setloglevel(args["log_level"])
    info_dump(args, 20)
    return args


def coroutine(func):
    @functools.wraps(func)
    def start(*args, **kwargs):
        cr=func(*args, **kwargs)
        cr.next()
        return cr
    return start

def jstdout(jo):
    sys.stdout.write(json.dumps(jo)+"\n")
    sys.stdout.flush()

def stdout(s):
    sys.stdout.write(s+"\n")
    sys.stdout.flush()
    
def dnorm(d):
    """
    Normalize dictionary
    
    >>> dnorm({"SoMeKeY":"  spaces  "})
    {'somekey': 'spaces'}
    """
    r={}
    for e in d:
        try:    r[e.lower()]=d[e].strip()
        except: r[e.lower()]=d[e]
    return r


def dstrip(d):
    """
    Strip each element in a dict
    
    >>> dstrip({"e1":" v1", "e2":" v2 "})
    {'e1': 'v1', 'e2': 'v2'}
    """
    for e in d:
        try:    d[e]=d[e].strip()
        except: pass
    return d


def move(src_path, dst_path):
    try:
        shutil.move(src_path, dst_path)
        return ("ok", None)
    except Exception, e:
        return ("error", e)


def retry(f, always=True, min_wait=1, max_wait=30, max_retries=10, logmsg=None):
    """
    Retries function 'f' : the function should throw an exception to indicate failure 
    
    :param always: boolean, should always retry true/false
    :param min_wait: int, minimum seconds between tries
    :param max_wait: int, maximum seconds before restarting cycle
    :param max_retries: int, maximum number of retries when 'always==False'
    
    @raise KeyboardInterrupt
    @return f()
    """
    showed_msg=False
    wait=min_wait
    retry_count=max_retries
    while True:
        try:
            return f()
        except KeyboardInterrupt:
            raise
        except Exception, exp:   
            if not showed_msg:
                showed_msg=True
                if logmsg is not None:
                    logging.warning(logmsg)     
            retry_count=max(retry_count-1, 0)
            if not always and retry_count==0:
                raise exp
            
        sleep(wait)
        
        wait=wait*2
        wait=wait if wait<max_wait else min_wait



if __name__=="__main__":
    import doctest
    doctest.testmod()
